@@ -699,8 +699,6 @@ mail_paned_view_constructed (GObject *object)
 	gtk_scrolled_window_set_policy (
 		GTK_SCROLLED_WINDOW (widget),
 		GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-	gtk_scrolled_window_set_shadow_type (
-		GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 	priv->scrolled_window = g_object_ref (widget);
 	gtk_paned_pack1 (GTK_PANED (container), widget, TRUE, FALSE);
 	gtk_widget_show (widget);
