@@ -1792,13 +1792,13 @@ em_subscription_editor_init (EMSubscriptionEditor *editor)
 	editor->priv->notebook = widget;
 	gtk_widget_show (widget);
 
-	g_object_bind_property (
+	e_binding_bind_property (
 		editor->priv->combo_box, "active",
 		editor->priv->notebook, "page",
 		G_BINDING_BIDIRECTIONAL |
 		G_BINDING_SYNC_CREATE);
 
-	g_signal_connect (
+	e_signal_connect_notify (
 		widget, "notify::sensitive",
 		G_CALLBACK (emse_notebook_sensitive_changed_cb), editor);
 
