@@ -590,9 +590,12 @@ do_ask (const gchar *text,
 	if (is_create_edit_add) {
 		gtk_dialog_add_buttons (
 			GTK_DIALOG (dialog),
-			_("_Cancel"), GTK_RESPONSE_CANCEL,
-			_("_Edit"), GTK_RESPONSE_YES,
-			_("_New"), GTK_RESPONSE_NO,
+			/* Translators: Dialog button to Cancel edit of an existing event/memo/task */
+			C_("mail-to-task", "_Cancel"), GTK_RESPONSE_CANCEL,
+			/* Translators: Dialog button to Edit an existing event/memo/task */
+			C_("mail-to-task", "_Edit"), GTK_RESPONSE_YES,
+			/* Translators: Dialog button to create a New event/memo/task */
+			C_("mail-to-task", "_New"), GTK_RESPONSE_NO,
 			NULL);
 	}
 
@@ -824,7 +827,9 @@ do_manage_comp_idle (struct _manage_comp *mc)
 }
 
 typedef struct {
+	EClientCache *client_cache;
 	ESource *source;
+	const gchar *extension_name;
 	ECalClientSourceType source_type;
 	CamelFolder *folder;
 	GPtrArray *uids;
@@ -840,8 +845,8 @@ do_mail_to_event (AsyncData *data)
 	GPtrArray *uids = data->uids;
 	GError *error = NULL;
 
-	client = e_cal_client_connect_sync (
-		data->source, data->source_type, NULL, &error);
+	client = e_client_cache_get_client_sync (data->client_cache,
+		data->source, data->extension_name, 30, NULL, &error);
 
 	/* Sanity check. */
 	g_return_val_if_fail (
@@ -1036,6 +1041,7 @@ do_mail_to_event (AsyncData *data)
 	g_ptr_array_unref (uids);
 	g_object_unref (folder);
 
+	g_object_unref (data->client_cache);
 	g_object_unref (data->source);
 	g_free (data->selected_text);
 	g_free (data);
@@ -1197,7 +1203,9 @@ mail_to_event (ECalClientSourceType source_type,
 
 		/* Fill the elements in AsynData */
 		data = g_new0 (AsyncData, 1);
+		data->client_cache = g_object_ref (e_shell_get_client_cache (shell));
 		data->source = g_object_ref (source);
+		data->extension_name = extension_name;
 		data->source_type = source_type;
 		data->folder = e_mail_reader_ref_folder (reader);
 		data->uids = g_ptr_array_ref (uids);
