@@ -747,6 +747,11 @@ shell_window_create_shell_view (EShellWindow *shell_window,
 	widget = GTK_WIDGET (e_shell_view_get_shell_taskbar (shell_view));
 	gtk_notebook_append_page (notebook, widget, NULL);
 
+	e_binding_bind_property (
+		widget, "height-request",
+		shell_window->priv->tooltip_label, "height-request",
+		G_BINDING_SYNC_CREATE);
+
 	/* Listen for changes that affect the shell window. */
 
 	e_signal_connect_notify_swapped (
@@ -1101,6 +1106,23 @@ e_shell_window_get_shell (EShellWindow *shell_window)
 	return E_SHELL (shell_window->priv->shell);
 }
 
+/**
+ * e_shell_window_is_main_instance:
+ * @shell_window: an #EShellWindow
+ *
+ * Returns, whether the @shell_window is the main instance, which is
+ * the window which was created as the first @shell_window.
+ *
+ * Returns: whether the @shell_window is the main instance
+ **/
+gboolean
+e_shell_window_is_main_instance (EShellWindow *shell_window)
+{
+	g_return_val_if_fail (E_IS_SHELL_WINDOW (shell_window), FALSE);
+
+	return shell_window->priv->is_main_instance;
+}
+
 /**
  * e_shell_window_get_shell_view:
  * @shell_window: an #EShellWindow
